from .utils import import_class


class Connection(object):

    def __init__(self, options):
        adaptor_name = options.get("adaptor")

        adaptor_class = import_class(adaptor_name)

        self.adaptor = adaptor_class(options)

    def __getattr__(self, name):
        return getattr(self.adaptor, name)



from .utils import import_class


class Device(object):

    def __init__(self, options, connection):
        self.name = options.get("name", "")
        self.connection = connection

        driver_name = options.get("driver")

        driver_class = import_class(driver_name)

        self.driver = driver_class(options, connection)

    def __getattr__(self, name):
        return getattr(self.driver, name)

    def start_process(self):
        pass

    def on(self):
        pass



from .robot import Robot
from .utils import import_class


robots = {}


def robot(options):
    name = options["name"]

    if name in robots:
        raise Exception("The robot's name must be unique")

    robot = Robot(options)

    robots[name] = robot

    return robot


def api(class_path, options):
    api_class = import_class(class_path)

    api_instance = api_class(options)

    return api_instance



class Adaptor(object):

    def __init__(self, options):
        self.name = options.get("name", "")
        self.host = options.get("host", None)
        self.port = options.get("port", None)

    def serialize(self):
        return {
            "name": self.name,
            "host": self.host,
            "port": self.port,
        }

    def connect(self):
        raise Exception("This needs to be overriden in child classes")

    def disconnect(self):
        raise Exception("This method needs to be overidden in child classes")



class Driver(object):

    def __init__(self, options, connection):
        self.name = options.get("name", "")
        self.connection_name = options.get("connection", "")
        self.pin = options.get("pin", None)
        self.interval = options.get("interval", 10)

        self.connection = connection

        self.commands = []
        self.events = []

    def start(self):
        raise Exception("This needs to be implemented by the child class")

    def halt(self):
        raise Exception("This needs to be implemented by the child class")

    def serialize(self):
        return {
            "name": self.name,
            "pin": self.pin,
            "connection": self.connection_name,
            "commands": self.commands,
            "events": self.events,
        }
